'''
Make the accumulated_* text files for plotting later with gnuplot for VNN-COMP.

This takes in a command line argument (directory with data files) and produces the accumulated files in the current dir.
Stanley Bak, July 2020
'''

import os
import sys

def main():
    "creates gnuplot input files ('accumulated_*') from data files in current directory for vnncomp"

    if len(sys.argv) != 2:
        print('Expected single argument: <data file directory>')
        sys.exit(1)

    directory = sys.argv[1]
    assert os.path.isdir(directory), "Argument should be an existing directory containing the data files."

    acas_all_dict = {'prefix': 'acasxu-all-', 'max_time': 300, 'num_benchmarks': 186}
    acas_hard_dict = {'prefix': 'acasxu-hard-', 'max_time': 6 * 60 * 60, 'num_benchmarks': 10}
    pat_dict = {'prefix': 'pat-all-', 'max_time': 900, 'num_benchmarks': 150}
    ggn_dict = {'prefix': 'ggn-all-', 'max_time': 300, 'num_benchmarks': 337}
    oval_dict = {'prefix': 'oval-dataset-', 'max_time': 3600, 'num_benchmarks': 300}
    neel_dict = {'prefix': 'neel-all-', 'max_time': 900, 'num_benchmarks': 128}

    all_dicts = [acas_all_dict, acas_hard_dict, pat_dict, ggn_dict, oval_dict, neel_dict]

    # omit files with these substrings
    omit_strings = ['empty', 'front', 'accumulated', 'results']
    
    output_prefix = 'accumulated-'
    min_secs = 0.11

    num_files_processed = 0

    for filename in sorted(os.listdir(directory)):
        if any([omit in filename for omit in omit_strings]):
            continue

        if not filename.endswith('.txt'):
            continue

        filepath = os.path.join(directory, filename)

        if not os.path.isfile(filepath):
            continue

        data_dict = None

        for cur_data_dict in all_dicts:
            if cur_data_dict['prefix'] in filename:
                data_dict = cur_data_dict

        if data_dict is None:
            print(f"Warning: data file with unknown prefix: {filename}")
            continue

        num_files_processed += 1

        max_time = data_dict['max_time']
        num_benchmarks = data_dict['num_benchmarks']
            
        print(f"Processing {filename}... ", end='', flush=True)

        with open(filepath, 'r') as fin:
            lines = fin.readlines()

        times = []

        for linenum, line in enumerate(lines):
            line = line.strip()

            if len(line) == 0:
                continue

            parts = line.split()

            secs = parts[0]

            if secs == "-":
                continue

            try:
                secs_float = float(secs)
                times.append(max(secs_float, min_secs))
            except ValueError:
                if linenum != 0: # line 0 may be the columns titles
                    print(f"\nWARNING: Skipping line {filename}:{linenum+1} (not a number): {line}")

        times.sort()

        with open(f"{output_prefix}{filename}", "w") as fout:
            for i, s in enumerate(times):
                fout.write(f"{s}\t{i+1}\n")

            if len(times) < num_benchmarks:
                fout.write(f"{max_time}\t{len(times)}") # last entry at max time if not all solved

        print(f"read {len(times)} entries")

    if num_files_processed == 0:
        print("Warning: no files were processed (did the passed in directory contain data files?)")

if __name__ == '__main__':
    main()
